@@ -9,6 +9,8 @@
 #' @param thin An integer to set the thinning interval range. If thin = 1,
 #' every iteration of the Gibbs-sampling chain will be kept. For highly autocorrelated
 #' chains, that are only examined by few iterations (say less than 1000).
+#' @param pvalue A numeric between 0 and 1 denoting the threshold of p-values a variable in the imputation
+#' model should not exceed. If they do, they are excluded from the imputation model.
 #' @param rounding_degrees A numeric vector with the presumed rounding degrees.
 #' @return A list with 1. 'y_ret' the n x 1 data.frame with the original and imputed values.
 #' 2. 'Sol' the Gibbs-samples for the fixed effects parameters.
@@ -18,6 +20,7 @@ imp_count_single <- function(y_imp,
                        nitt = 22000,
                        burnin = 2000,
                        thin = 20,
+                       pvalue = 0.2,
                        rounding_degrees = c(1, 10, 100, 1000)){
 
   # ----------------------------- preparing the X data ------------------
@@ -25,45 +28,91 @@ imp_count_single <- function(y_imp,
   X_imp <- cleanup(X_imp)
 
   # standardize X
-  X_imp_stand <- stand(X_imp, rounding_degrees = rounding_degrees)
+  X <- stand(X_imp, rounding_degrees = rounding_degrees)
 
   #the missing indactor indicates, which values of y are missing.
   missind <- is.na(y_imp)
 
   n <- length(y_imp)
 
-  #starting model
-  ph <- sample_imp(y_imp)[, 1]
-  tmp_0_all <- data.frame(target = ph)
-  xnames_0 <- paste("X", 1:ncol(X_imp_stand), sep = "")
-  tmp_0_all[xnames_0] <- X_imp_stand
-  tmp_0_sub <- tmp_0_all[!missind, , drop = FALSE]
+  # ----------- set up a maximal model matrix with all possible relevant (dummy) variables -----
+  # In the imputation model only actually relevant (dummy) variables shall be present.
+  # THis is done by setting up a mirror of the initial model matrix.
+  # Then step by step this model matrix is reduced to all actually relevant (dummy) variables.
+  # This reduction is based on models using the observed data.
+  # The last step prior to the imputation-parameters estimation is to restrict the initial mode matrix
+  # to those variables, left in the reduced mirror model matrix.
 
-  reg_1_all <- stats::glm(target ~ 0 + ., data =  tmp_0_all, family = "poisson")
-  reg_1_sub <- stats::glm(target ~ 0 + ., data =  tmp_0_sub, family = "poisson")
+  #define a place holder (ph)
+  ph <- sample_imp(y_imp)[, 1]
+
+  tmp_0_all <- data.frame(target = ph, X)
+  xnames_1 <- colnames(X)
+
+  tmp_formula <- paste("target~ 0 + ", paste(xnames_1, collapse = "+"), sep = "")
+  reg_1_all <- stats::glm(stats::formula(tmp_formula), data = tmp_0_all,
+                          family = "poisson")
 
   X_model_matrix_1_all <- stats::model.matrix(reg_1_all)
   xnames_1 <- paste("X", 1:ncol(X_model_matrix_1_all), sep = "")
+  colnames(X_model_matrix_1_all) <- xnames_1
+
+  tmp_0_all <- data.frame(target = ph)
+  tmp_0_all[, xnames_1] <- X_model_matrix_1_all
+
+  #From this initial model matrix X_model_matrix_1_all
+  #now step by step irrelavant variables are removed.
+  X_model_matrix_1_sub <- X_model_matrix_1_all[!missind, , drop = FALSE]
+
+
+  #first step of the reduction is to remove variables having a NA-effect (e.g. due to colinearity) on y
+  #tmp_1 shall include the covariates (like X_model_matrix) and additionally the target variable
+  ph_sub <- y_imp[!missind]
+  tmp_1_sub <- data.frame(target = ph_sub)
+  xnames_1 <- colnames(X_model_matrix_1_sub)
+  tmp_1_sub[, xnames_1] <- X_model_matrix_1_sub
+
+  tmp_formula <- paste("target~ 0 + ", paste(xnames_1, collapse = "+"), sep = "")
+
+  reg_1_sub <- stats::glm(stats::formula(tmp_formula), data = tmp_1_sub,
+                          family = "poisson")
 
   #remove unneeded variables
-  unneeded <- is.na(stats::coefficients(reg_1_sub))
-  xnames_2 <- xnames_1[!unneeded]
+  X_model_matrix_1_sub <- X_model_matrix_1_sub[, !is.na(stats::coefficients(reg_1_sub)),
+                                               drop = FALSE]
 
-  tmp_2_all <- data.frame(target = y_imp)
-  X_model_matrix_2_all <- X_model_matrix_1_all[, !unneeded, drop = FALSE]
+  # Remove insignificant variables from the imputation model
+  check <- TRUE
+  while(check){
+    tmp_1_sub <- data.frame(target = ph_sub)
+    xnames_1 <- colnames(X_model_matrix_1_sub)
+    tmp_1_sub[, xnames_1] <- X_model_matrix_1_sub
+    tmp_formula <- paste("target~ 0 + ", paste(xnames_1, collapse = "+"), sep = "")
+    reg_1_sub <- stats::glm(stats::formula(tmp_formula), data = tmp_1_sub,
+                            family = "poisson")
 
-  tmp_2_all[, xnames_2] <- X_model_matrix_2_all
+    pvalues <- summary(reg_1_sub)$coefficients[, 4]
+    insignificant_variables <- which(pvalues > pvalue)
+    most_insignificant <- insignificant_variables[which.max(pvalues[insignificant_variables])]
 
-  tmp_2_sub <- tmp_2_all[!missind, , drop = FALSE]
+    if(length(most_insignificant) == 0){
+      check <- FALSE
+    }else{
+      X_model_matrix_1_sub <- stats::model.matrix(reg_1_sub)[, -most_insignificant, drop = FALSE]
+    }
+  }
+
+  tmp_2_all <- tmp_0_all[, colnames(tmp_1_sub), drop = FALSE]
+  tmp_2_all$target[missind] <- NA
 
   # -------------- calling the gibbs sampler to get imputation parameters----
 
-  fixformula <- stats::formula(paste("target~", paste(xnames_2, collapse = "+"), "- 1",
+  fixformula <- stats::formula(paste("target~", paste(xnames_1, collapse = "+"), "- 1",
                                      sep = ""))
 
   prior <- list(R = list(V = 1e-07, nu = -2))
 
-  MCMCglmm_draws <- MCMCglmm::MCMCglmm(fixformula, data = tmp_2_all,
+  MCMCglmm_draws <- MCMCglmm::MCMCglmm(fixformula, data = tmp_1_sub,
                                        verbose = FALSE, pr = TRUE, prior = prior,
                                        family = "poisson",
                                        saveX = TRUE,
@@ -72,7 +121,7 @@ imp_count_single <- function(y_imp,
                                        burnin = burnin)
 
   pointdraws <- MCMCglmm_draws$Sol
-  xdraws <- pointdraws[, 1:ncol(X_model_matrix_2_all), drop = FALSE]
+  xdraws <- pointdraws[, 1:ncol(X_model_matrix_1_sub), drop = FALSE]
   variancedraws <- MCMCglmm_draws$VCV
   # the last column contains the variance (not standard deviation) of the residuals
 
@@ -82,12 +131,12 @@ imp_count_single <- function(y_imp,
   # -------------------- drawing samples with the parameters from the gibbs sampler --------
   ###start imputation
 
-  fix.eff.imp <- matrix(xdraws[select.record, ], nrow = ncol(X_model_matrix_2_all))
+  fix.eff.imp <- matrix(xdraws[select.record, ], nrow = ncol(X_model_matrix_1_sub))
 
   sigma.y.imp <- sqrt(variancedraws[select.record, ncol(variancedraws)])
 
 
-  lambda <- exp(stats::rnorm(n, X_model_matrix_2_all %*% fix.eff.imp, sigma.y.imp))
+  lambda <- exp(stats::rnorm(n, as.matrix(tmp_2_all[, xnames_1, drop = FALSE]) %*% fix.eff.imp, sigma.y.imp))
 
   # note: the maximum lambda is something about 2.14e9
   if(max(lambda) > 2.14e9) {
